GLOBAL.setmetatable(env,{__index=function(t,k) return GLOBAL.rawget(GLOBAL,k) end})

local hasSWPrefabs = IsDLCEnabled(3) or IsDLCEnabled(2)

if not hasSWPrefabs then return end

local function PackimsOnAttack(inst, attacker, target)
    if target.components.burnable and not target.components.burnable:IsBurning() then
        if target.components.freezable and target.components.freezable:IsFrozen() then          
            target.components.freezable:Unfreeze()
        end
    end

    if target.components.freezable then
        target.components.freezable:AddColdness(-1)
        if target.components.freezable:IsFrozen() then
            target.components.freezable:Unfreeze()
        end
    end

    if target.components.sleeper and target.components.sleeper:IsAsleep() then
        target.components.sleeper:WakeUp()
    end

    attacker.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")

    if target:HasTag("aquatic") then
        local pt = target:GetPosition()
        local smoke = SpawnPrefab("smoke_out")

        smoke.Transform:SetPosition(pt:Get())

        if target.SoundEmitter then 
            target.SoundEmitter:PlaySound("dontstarve_DLC002/common/fire_weapon_out") 
        end
    else
        local tauntfx = SpawnPrefab("tauntfire_fx")
        local pos = target:GetPosition()

        tauntfx.Transform:SetPosition(pos.x, pos.y + 0.7, pos.z)
        tauntfx:DoTaskInTime(0.43, tauntfx.Remove)
    end

    target:PushEvent("attacked", {attacker = attacker, damage = TUNING.PIG_DAMAGE})
end

local no_target_tags = {"FX", "NOCLICK","INLIMBO", "abigail", "player"}

local function RetargetFindValidator(inst)
    return function(guy)
        if (
            not guy.LightWatcher or guy.LightWatcher:IsInLight()
        ) then
            return 
                        guy.components.health
                and     not guy.components.health:IsDead()
                and     guy.components.combat
                and     inst.components.combat:CanTarget(guy)
                and (
                    guy.components.combat.target == GetPlayer()
                    or GetPlayer().components.combat.target == guy
                    or guy.components.combat.target == inst
                )
        end
    end
end

local function RetargetFn(inst)
    if not inst.last_fire_time or (inst.fire_interval and (GetTime() - inst.last_fire_time) > inst.fire_interval) then
        return FindEntity(inst, TUNING.PIG_TARGET_DIST, RetargetFindValidator(inst), nil, no_target_tags)
    end
    
    return false
end

local function TryBurnItems(container)
    for i = 1, container:GetNumSlots() do
        local item = container:GetItemInSlot(i)
        if item then 
            local replacement = nil 

            if item.components.cookable then 
                replacement = item.components.cookable:GetProduct()

            elseif item.components.edible and item.components.edible.foodtype == "WOOD" then
                replacement = "charcoal"

            elseif item.components.burnable and item.prefab ~= "coffeebeans_cooked" then
                replacement = "ash"
            end

            if replacement then 
                local stacksize = 1

                if item.components.stackable then 
                    stacksize = item.components.stackable:StackSize()
                end

                local newprefab = SpawnPrefab(replacement)
                
                if newprefab.components.stackable then 
                    newprefab.components.stackable:SetStackSize(stacksize)
                end 
                
                if newprefab.components.perishable and item.components.perishable and not newprefab:HasTag("smallcreature") then
                    local new_percent = 1 - (1 - item.components.perishable:GetPercent())*.5
                    newprefab.components.perishable:SetPercent(new_percent)
                end

                container:RemoveItemBySlot(i)
                item:Remove()
                container:GiveItem(newprefab, i)
            end
        end
    end
end

local function CheckForChanges(inst)
    if inst.PackimState == "FIRE" then
        local fireStaff = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)

        if fireStaff then
            fireStaff.components.weapon:SetOnAttack(PackimsOnAttack)
            fireStaff.components.weapon:SetDamage(TUNING.PIG_DAMAGE)
        end
    end
end

------------------------------------------------------------------------------------

AddPrefabPostInit("packim", function(inst)
    local _CheckFireTransform = inst.checkfiretransform
    local _OnPreLoad = inst.OnPreLoad
    local _TryEat = inst.tryeat

    inst.components.combat:SetRetargetFunction(3, RetargetFn)

    inst.checkfiretransform = function(inst)
        local ret = _CheckFireTransform(inst)
        CheckForChanges(inst)

        return ret
    end

    inst.OnPreLoad = function(inst, data)
        _OnPreLoad(inst, data)
        CheckForChanges(inst)
    end

    inst.tryeat = function(inst)
        if inst.PackimState == "FIRE" then
            TryBurnItems(inst.components.container)
            return false
        end
        
        return _TryEat(inst)
    end
end)

------------------------------------------------------------------------------------

require "behaviours/runaway"

local RUN_START_DIST = 4
local RUN_STOP_DIST = 5.5

local function GetHunter(inst)
    return FindEntity(inst, RUN_START_DIST,
        function(guy) return guy.components.combat and guy.components.combat.target == inst end,
        nil, {'notarget'})
end

AddBrainPostInit("packimbrain", function(brain)
    local children = brain.bt.root.children

    local follow_node = nil
    local follow_index = nil

    for i, node in pairs(children) do
        if node.name == "Follow" then
            follow_index = i
            follow_node = node
        end
    end

    if follow_node then
        table.remove(children, follow_index)
        table.insert(children, 2, follow_node)
    end
    
    table.insert(children, 3, RunAway(brain.inst, GetHunter, RUN_START_DIST, RUN_STOP_DIST))
end)
